/*
 * Copyright 
 *   2007 axYus - www.axyus.com
 *   2007 N.Le Corre - nicolas.lecorre@axyus.com
 * 
 * This file is part of XEMELIOS.
 * 
 * XEMELIOS is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * XEMELIOS is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with XEMELIOS; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package fr.gouv.finances.cp.xemelios.controls.paye;

import java.util.Hashtable;
import java.util.Vector;

import org.apache.log4j.Logger;
import org.xml.sax.Attributes;
import org.xml.sax.Locator;
import org.xml.sax.SAXException;

import fr.gouv.finances.cp.xemelios.controls.AbstractUnitControl;
import fr.gouv.finances.cp.xemelios.controls.Anomalie;
import fr.gouv.finances.cp.xemelios.controls.IdGenerator;
import fr.gouv.finances.cp.xemelios.controls.Node;
import java.util.ArrayList;
import java.util.TreeSet;

public class PAY16 extends AbstractUnitControl {
	private final static Logger logger = Logger.getLogger(PAY16.class);
    public static final transient String CTRL_ID = PAY16.class.getSimpleName();
	private Vector<Anomalie> anos = new Vector<Anomalie>();
	private String docId = null;

	
//	private static final String ON_RETURN_ANOMALIES = "DocumentPaye/DonneesIndiv/PayeIndivMensuel/";
	private static final String ON_RETURN_ANOMALIES = "DocumentPaye/DonneesIndiv/";
//	private static final String CHEMIN_PK_ANNEE = "DocumentPaye/Annee/";
//	private static final String CHEMIN_PK_MOIS = "DocumentPaye/Mois/";
	private static final String CHEMIN_PAYE = "DocumentPaye/DonneesIndiv/PayeIndivMensuel/";
//	private static final String CHEMIN_PK_MATRICULE = "DocumentPaye/DonneesIndiv/PayeIndivMensuel/Agent/Matricule/";
//	private String pk_annee = "", pk_mois = "", pk_matricule = "";
//	private String payeNodeId = "";

	private static final String[] typesLigneDevantEtrePresent = {"TraitBrut","IndemResid","SupFam","AvantageNature","Indemnite","RemDivers","Deduction","Acompte","Rappel","Retenue","Cotisation"};

	private static final String CHEMIN_TYPES_LIGNES = "DocumentPaye/DonneesIndiv/PayeIndivMensuel/Remuneration/[a-zA-Z]*/";
	private static final String REPLACE_PAYE = "##PAYE##";
	private static final String REPLACE_LISTE = "##LISTE##";
	private TreeSet<String> lignes = new TreeSet<String>();
	
	public void startDocument() throws SAXException {
		logger.info("In "+CTRL_ID);
	}
	
	public void setDocId(String docID) {
		this.docId = docID;		
	}

	public void startElement(String uri, String localName, String qName, Attributes atts, String xpath) throws SAXException {
		if (xpath.endsWith(CHEMIN_PAYE)) {
			anos = new Vector<Anomalie>();
//			payeNodeId = atts.getValue("ano:node-id");
//			pk_matricule = "";
			lignes = new TreeSet<String>();
//		} else if (xpath.endsWith(CHEMIN_PK_ANNEE)) {
//			pk_annee = atts.getValue("V");
//		} else if (xpath.endsWith(CHEMIN_PK_MOIS)) {
//			pk_mois = atts.getValue("V");
//		} else if (xpath.endsWith(CHEMIN_PK_MATRICULE)) {
//			pk_matricule = atts.getValue("V");
		} else if (xpath.matches(CHEMIN_TYPES_LIGNES)) {
			lignes.add(localName);
		}
	}

	public Vector<Anomalie> endElement(String uri, String localName, String qName, String xpath) throws SAXException {
		if (xpath.endsWith(ON_RETURN_ANOMALIES)) {
			// TEST
			//String liste = "";
                    ArrayList<String> liste = new ArrayList<String>();
			for (String s:typesLigneDevantEtrePresent) {
				if (!lignes.contains(s)) liste.add(s);
			}
//			if (liste.endsWith(", ")) liste = liste.substring(0, liste.length()-2);
			// ANOMALIE
			if (liste.size()>0) {
                            StringBuilder sb = new StringBuilder();
                            sb.append("<ul>");
                            for(String s:liste) sb.append("<li>").append(s).append("</li>");
                            sb.append("</ul>");
				String msg = getDocumentModel().getControlById(CTRL_ID).getMessage().getHtmlContent();
//				msg = msg.replaceAll(REPLACE_PAYE, pk_annee+" / "+pk_mois+" / "+pk_matricule);
				msg = msg.replaceAll(REPLACE_LISTE, sb.toString());
//				String path="@added:primary-key='"+pk_annee+"-"+pk_mois+"-"+pk_matricule+"'";
				String anoId = IdGenerator.nextId();
				String libelleLien = "";
				Hashtable<String, Object> xslParams = new Hashtable<String, Object>();
				xslParams.put("elementId", "Paye");
				xslParams.put("browser-destination", "internal");
				xslParams.put("anoId", anoId);
				Anomalie ano = new Anomalie(anoId,
											getDocumentModel().getControlById(CTRL_ID).getId(),
											getDocumentModel().getControlById(CTRL_ID).getLibelle(),
											getDocumentModel().getDocumentId(),
											"etatPaye",
											"payeIndivMensuel", // lment importable
											"", // id lment importable
											localName,
											libelleLien,
											msg,
											getDocumentModel().getControlById(CTRL_ID).getRegle().getHtmlContent(),
											getDocumentModel().getControlById(CTRL_ID).getSeverity(),
											null, // xpath vers le bordereau sous lequel se trouve l'anomalie
											xslParams); // paramtres supplmentaires pour accder  l'lment en anomalie
//				ano.addNode(new Node(payeNodeId));
				anos.add(ano);
			}
			return anos;
		} else {
			return null;
		}
	}

	public void characters(char[] ch, int start, int length, String xpath) throws SAXException { }

	public void endDocument() throws SAXException { }

	public void endPrefixMapping(String prefix, String xpath) throws SAXException { }

	public void ignorableWhitespace(char[] ch, int start, int length, String xpath) throws SAXException { }

	public void processingInstruction(String target, String data, String xpath) throws SAXException { }

	public void setDocumentLocator(Locator locator) { }

	public void setParameters(Hashtable<String, Object> params) { }

	public void skippedEntity(String name, String xpath) throws SAXException { }

	public void startPrefixMapping(String prefix, String uri, String xpath) throws SAXException {}

}
