<?xml version="1.0" encoding="ISO-8859-1"?>
<!DOCTYPE xsl:stylesheet [
<!ENTITY nbsp "&#160;">
<!ENTITY egrave "&#232;">
<!ENTITY euro "&#8364;">
<!ENTITY agrave "&#224;">
<!ENTITY eacute "&#233;">
]>
<xsl:transform xmlns:xsl="http://www.w3.org/1999/XSL/Transform" xmlns="http://www.xemelios.org/namespaces#cge" xmlns:n="http://www.xemelios.org/namespaces#cge" xmlns:added="http://www.xemelios.org/namespaces#added" version="2.0">
    <xsl:character-map name="accents">
        <xsl:output-character character="" string="&amp;#224;"/>
        <xsl:output-character character="" string="&amp;#233;"/>
        <xsl:output-character character="" string="&amp;#232;"/>
        <xsl:output-character character="" string="&amp;#234;"/>
        <xsl:output-character character="" string="&amp;#235;"/>
        <xsl:output-character character="" string="&amp;#238;"/>
        <xsl:output-character character="" string="&amp;#239;"/>
        <xsl:output-character character="" string="&amp;#244;"/>
        <xsl:output-character character="" string="&amp;#249;"/>
    </xsl:character-map>

    <xsl:decimal-format name="decformat" decimal-separator="," grouping-separator=" " digit="#" pattern-separator=";" NaN="NaN" minus-sign="-"/>
    
    <!-- Inclusion des XSL externes -->
    <xsl:include href="./common-xsl/Style.xsl"/>
    <xsl:include href="./common-xsl/Error.xsl"/>
    <xsl:include href="./common-xsl/Navigate.xsl"/>
    <xsl:include href="./common-xsl/Header.xsl"/>
    <xsl:include href="./common-xsl/Number.xsl"/>
    
    <!-- Paramtres d'entre -->
    <xsl:param name="show.formulaire"/>
    <xsl:param name="browser-destination"/>
    
    <!-- Paramtres d'entre de la liste de rsultat -->
    <xsl:param name="libPoste">0</xsl:param>
    
    <!-- Paramtres d'erreur -->
    <xsl:param name="error.message">nomessage</xsl:param>

    <!-- Paramtres d'entre de la liste de rsultat -->
    <xsl:param name="numCompte">0</xsl:param>
    <xsl:param name="comptePEC">0</xsl:param>
    <xsl:param name="ligneRecette">0</xsl:param>

    <xsl:param name="type.treated" select="tokenize(name(/n:CompteGestionEtat//*[contains(name(),'Page')]),'Page')[2]"/>

    <xsl:output method="xhtml" indent="yes" use-character-maps="accents" encoding="ISO-8859-1"/>

    <!-- pour eviter les sorties parasites de tags non matches -->
    <xsl:template match="*"/>
  <!-- Variables -->
    <xsl:variable name="page-format" select="paysage"/>
    
    
    
    <xsl:variable name="Entete">
        <xsl:copy-of  select="/n:CompteGestionEtat/n:Entete"/>
    </xsl:variable>

    <xsl:template match="/n:CompteGestionEtat">
        <xsl:variable name="NumPage">
            <xsl:choose>
                <xsl:when test="$type.treated eq 'DeveloppementRecette'">
                    <xsl:value-of select="/n:CompteGestionEtat/n:DeveloppementRecette/n:PageDeveloppementRecette/n:Pied/@NumPage"/>
                </xsl:when>
                <xsl:when test="$type.treated eq 'Recapitulation'">
                    <xsl:value-of select="/n:CompteGestionEtat/n:Recapitulation/n:PageRecapitulation/n:Pied/@NumPage"/>
                </xsl:when>
                <xsl:when test="$type.treated eq 'Reversement'">
                    <xsl:value-of select="/n:CompteGestionEtat/n:Reversement/n:PageReversement/n:Pied/@NumPage"/>
                </xsl:when>
            </xsl:choose>
        </xsl:variable>
        
        <html>
            <head>
                <xsl:choose>
                    <xsl:when test="$type.treated eq 'DeveloppementRecette'">
                        <title>D&#233;veloppement des recettes budg&#233;taires et des comptes sp&#233;ciaux</title>
                    </xsl:when>
                    <xsl:when test="$type.treated eq 'Recapitulation'">
                        <title>R&#233;capitulation des recettes budg&#233;taires et des comptes sp&#233;ciaux</title>
                    </xsl:when>
                    <xsl:when test="$type.treated eq 'Reversement'">
                        <title>Reversemments de fonds sur minist&#232;res &#224; annuler</title>
                    </xsl:when>
                </xsl:choose>   
                <xsl:call-template name="style"/>                
            </head>
            <body>
                <xsl:if test="$error.message!='nomessage'">
                    <xsl:attribute name="onload">javascript:showDialog('<xsl:value-of select="$error.message"/>');</xsl:attribute>
                </xsl:if>
                <xsl:call-template name="Error"><!--xsl:with-param name="error.message" select="$error.message"/--></xsl:call-template>
                <xsl:call-template name="navigate">
                    <xsl:with-param name="docId">compteGestionEtat</xsl:with-param>
                    <xsl:with-param name="etatId"><xsl:value-of select="$type.treated"/></xsl:with-param>
                    <xsl:with-param name="elementId">Page<xsl:value-of select="$type.treated"/></xsl:with-param>
                    <xsl:with-param name="sous.elementId">Ligne<xsl:value-of select="$type.treated"/></xsl:with-param>
                    <xsl:with-param name="NumPage" select="$NumPage"/>
                    <xsl:with-param name="Entete" select="$Entete"/>
                    <xsl:with-param name="show.formulaire" select="$show.formulaire"/>
                    <xsl:with-param name="show.formulaire.compte" select="1"/>
                    <xsl:with-param name="show.depense.recette" select="0"/>
                </xsl:call-template>
                <xsl:call-template name="header" >
                    <xsl:with-param name="Entete" select="$Entete"/>
                    <xsl:with-param name="Titre">
                        <xsl:choose>
                            <xsl:when test="$type.treated eq 'DeveloppementRecette'">D&#233;veloppement des recettes budg&#233;taires et des comptes sp&#233;ciaux</xsl:when>
                            <xsl:when test="$type.treated eq 'Recapitulation'">R&#233;capitulation des recettes budg&#233;taires et des comptes sp&#233;ciaux</xsl:when>
                            <xsl:when test="$type.treated eq 'Reversement'">Reversemments de fonds sur minist&#232;res &#224; annuler</xsl:when>
                        </xsl:choose>
                    </xsl:with-param>
                    <xsl:with-param name="Titre.Plus">
                        <xsl:choose>
                            <xsl:when test="$type.treated eq 'DeveloppementRecette' or $type.treated eq 'Reversement'"><h1>Ventilation par ligne</h1></xsl:when>
                            <xsl:when test="$type.treated eq 'Recapitulation'"><h1>R&#233;capitulation par compte budg&#233;taires</h1></xsl:when>
                        </xsl:choose>
                    </xsl:with-param>
                </xsl:call-template>
                <br/>
                <br/>
                <br/>
                <br/>
                <xsl:choose>
                    <xsl:when test="$type.treated eq 'DeveloppementRecette'">
                        <xsl:call-template name="mainTable">
                            <xsl:with-param name="el" select="/n:CompteGestionEtat/n:DeveloppementRecette/n:PageDeveloppementRecette"/>
                        </xsl:call-template>
                    </xsl:when>
                    <xsl:when test="$type.treated eq 'Recapitulation'">
                        <xsl:call-template name="mainTable">
                            <xsl:with-param name="el" select="/n:CompteGestionEtat/n:Recapitulation/n:PageRecapitulation"/>
                        </xsl:call-template>
                    </xsl:when>
                    <xsl:when test="$type.treated eq 'Reversement'">
                        <xsl:call-template name="mainTable">
                            <xsl:with-param name="el" select="/n:CompteGestionEtat/n:Reversement/n:PageReversement"/>
                        </xsl:call-template>
                    </xsl:when>
                </xsl:choose>
                <br/>
                <br/>
                <br/>
                <br/>
                <p>
                    <center>Page - <xsl:value-of select="$NumPage"/></center>
                </p>
                <br/>
                <br/>
                <center>__________________________________________</center>
                <xsl:call-template name="navigate">
                    <xsl:with-param name="docId">compteGestionEtat</xsl:with-param>
                    <xsl:with-param name="etatId"><xsl:value-of select="$type.treated"/></xsl:with-param>
                    <xsl:with-param name="elementId">Page<xsl:value-of select="$type.treated"/></xsl:with-param>
                    <xsl:with-param name="sous.elementId">Ligne<xsl:value-of select="$type.treated"/></xsl:with-param>
                    <xsl:with-param name="NumPage" select="$NumPage"/>
                    <xsl:with-param name="Entete" select="$Entete"/>
                    <xsl:with-param name="show.formulaire" select="$show.formulaire"/>
                    <xsl:with-param name="show.formulaire.compte" select="1"/>
                    <xsl:with-param name="show.depense.recette" select="0"/>
                    <xsl:with-param name="ancre">#footer</xsl:with-param>
                    <xsl:with-param name="display.logos.entete">0</xsl:with-param>
                </xsl:call-template>
                <p id="footer"/>
            </body>
        </html>
    </xsl:template>

    <xsl:template name="mainTable">
        <xsl:param name="el"/>
        <table width="100%" class="bordered" id="main">
            <colgroup>
                <col/>
                <col width="7%"/>
                <col width="7%"/>
                <col width="7%"/>
                <col width="7%"/>
                <col width="7%"/>
                <col width="7%"/>
                <col width="7%"/>
                <col width="7%"/>
                <col width="4%"/>
                <col width="4%"/>
                <col width="4%"/>                
            </colgroup>
            <thead>
                <tr  class="titre">
                    <td class="titre2 center bold bordered">Nature des produits</td>
                    <td class="titre2 center bold bordered">Restes A Recouver<br/>au 1<sup>er</sup> Janvier</td> 
                    <td class="titre2 center bold bordered">Droits pris<br/>en charge</td>
                    <td class="titre2 center bold bordered">Annulations de<br/>prise en charge</td>
                    <td class="titre2 center bold bordered">Total des droits<br/>constat&#233;s</td>
                    <td class="titre2 center bold bordered">Recouvrement sur<br/>prise en charge</td>
                    <td class="titre2 center bold bordered">Recettes<br/>au comptant</td>
                    <td class="titre2 center bold bordered">Restes A Recouver<br/>au 31 D&#233;cembre</td>
                    <td class="titre2 center bold bordered">Total des recettes<br/>budg&#233;taires</td>
                    <td class="titre2 center bold bordered">Ligne<br/>de<br/>recette</td>
                    <td class="titre2 center bold bordered">Compte<br/>de<br/>recette</td>
                    <td class="titre2 center bold bordered">Compte<br/>de<br/>P.E.C.</td>
                </tr>
            </thead>
            <tbody>
                <xsl:variable name="lignes">
                    <xsl:choose>
                        <xsl:when test="$type.treated eq 'DeveloppementRecette'"><xsl:copy-of select="$el/n:LigneDeveloppementRecette"/></xsl:when>
                        <xsl:when test="$type.treated eq 'Recapitulation'"><xsl:copy-of select="$el/n:LigneRecapitulation"/></xsl:when>
                        <xsl:when test="$type.treated eq 'Reversement'"><xsl:copy-of select="$el/n:LigneReversement"/></xsl:when>
                    </xsl:choose>
                </xsl:variable>
                
                <xsl:for-each select="$lignes/*">
                    <xsl:sort select="."/>
                    <tr>
                        <xsl:choose>
                            <xsl:when test="$numCompte != '0' and @NumCompte=$numCompte and @ComptePEC=$comptePEC and @LigneRecette=$ligneRecette">
                                <xsl:attribute name="id">highlighted</xsl:attribute>
                            </xsl:when>
                            <xsl:otherwise> 
                                <xsl:if test="upper-case(@Type) eq 'TOTAL' and number(@added:Padding) eq 0">
                                    <xsl:attribute name="class">total bordureTopBold bordureBottomBold</xsl:attribute>
                                </xsl:if>
                                <xsl:choose>                                    
                                    <xsl:when test="(position() mod 2) = 0">
                                        <xsl:attribute name="id">colorised1</xsl:attribute>
                                    </xsl:when>
                                    <xsl:otherwise>
                                        <xsl:attribute name="id">colorised2</xsl:attribute>
                                    </xsl:otherwise>
                                </xsl:choose>
                            </xsl:otherwise>
                        </xsl:choose>                        
                        <xsl:choose>
                            <xsl:when test="@RARDeb &gt; 0 or @Droits &gt; 0 or @Annulations &gt; 0 or @TotDroits &gt; 0 or @Recouvrement &gt; 0 or @RecettesComptant &gt; 0 or @RARFin &gt; 0 or @TotRecetteBudg &gt; 0">
                                <xsl:choose>
                                    <xsl:when test="upper-case(@Type) eq 'TOTAL' and number(@added:Padding) eq 0">
                                        <td class="bordureTopBold bordureBottomBold bordureRight"><xsl:attribute name="id">padding<xsl:value-of select="@added:Padding"/></xsl:attribute><xsl:value-of select="@NatureProduit"/></td>
                                        <td class="montant bordureTopBold bordureBottomBold bordureRight"><xsl:call-template name="number"><xsl:with-param name="num" select="@RARDeb"/><xsl:with-param name="hide.zero">1</xsl:with-param></xsl:call-template></td>
                                        <td class="montant bordureTopBold bordureBottomBold bordureRight"><xsl:call-template name="number"><xsl:with-param name="num" select="@Droits"/><xsl:with-param name="hide.zero">1</xsl:with-param></xsl:call-template></td>
                                        <td class="montant bordureTopBold bordureBottomBold bordureRight"><xsl:call-template name="number"><xsl:with-param name="num" select="@Annulations"/><xsl:with-param name="hide.zero">1</xsl:with-param></xsl:call-template></td>
                                        <td class="montant bordureTopBold bordureBottomBold bordureRight"><xsl:call-template name="number"><xsl:with-param name="num" select="@TotDroits"/><xsl:with-param name="hide.zero">1</xsl:with-param></xsl:call-template></td>
                                        <td class="montant bordureTopBold bordureBottomBold bordureRight"><xsl:call-template name="number"><xsl:with-param name="num" select="@Recouvrement"/><xsl:with-param name="hide.zero">1</xsl:with-param></xsl:call-template></td>
                                        <td class="montant bordureTopBold bordureBottomBold bordureRight"><xsl:call-template name="number"><xsl:with-param name="num" select="@RecettesComptant"/><xsl:with-param name="hide.zero">1</xsl:with-param></xsl:call-template></td>
                                        <td class="montant bordureTopBold bordureBottomBold bordureRight"><xsl:call-template name="number"><xsl:with-param name="num" select="@RARFin"/><xsl:with-param name="hide.zero">1</xsl:with-param></xsl:call-template></td>
                                        <td class="montant bordureTopBold bordureBottomBold bordureRight"><xsl:call-template name="number"><xsl:with-param name="num" select="@TotRecetteBudg"/><xsl:with-param name="hide.zero">1</xsl:with-param></xsl:call-template></td>
                                        <td class="right bordureTopBold bordureBottomBold bordureRight"><xsl:value-of select="@LigneRecette"/></td>
                                        <td class="right bordureTopBold bordureBottomBold bordureRight"><xsl:value-of select="@NumCompte"/></td>
                                        <td class="right bordureTopBold bordureBottomBold"><xsl:value-of select="@ComptePEC"/></td>
                                    </xsl:when>
                                    <xsl:otherwise>
                                        <td class="bordered"><xsl:attribute name="id">padding<xsl:value-of select="@added:Padding"/></xsl:attribute><xsl:value-of select="@NatureProduit"/></td>
                                        <td class="montant bordered"><xsl:call-template name="number"><xsl:with-param name="num" select="@RARDeb"/><xsl:with-param name="hide.zero">1</xsl:with-param></xsl:call-template></td>
                                        <td class="montant bordered"><xsl:call-template name="number"><xsl:with-param name="num" select="@Droits"/><xsl:with-param name="hide.zero">1</xsl:with-param></xsl:call-template></td>
                                        <td class="montant bordered"><xsl:call-template name="number"><xsl:with-param name="num" select="@Annulations"/><xsl:with-param name="hide.zero">1</xsl:with-param></xsl:call-template></td>
                                        <td class="montant bordered"><xsl:call-template name="number"><xsl:with-param name="num" select="@TotDroits"/><xsl:with-param name="hide.zero">1</xsl:with-param></xsl:call-template></td>
                                        <td class="montant bordered"><xsl:call-template name="number"><xsl:with-param name="num" select="@Recouvrement"/><xsl:with-param name="hide.zero">1</xsl:with-param></xsl:call-template></td>
                                        <td class="montant bordered"><xsl:call-template name="number"><xsl:with-param name="num" select="@RecettesComptant"/><xsl:with-param name="hide.zero">1</xsl:with-param></xsl:call-template></td>
                                        <td class="montant bordered"><xsl:call-template name="number"><xsl:with-param name="num" select="@RARFin"/><xsl:with-param name="hide.zero">1</xsl:with-param></xsl:call-template></td>
                                        <td class="montant bordered"><xsl:call-template name="number"><xsl:with-param name="num" select="@TotRecetteBudg"/><xsl:with-param name="hide.zero">1</xsl:with-param></xsl:call-template></td>
                                        <td class="bordered right"><xsl:value-of select="@LigneRecette"/></td>
                                        <td class="bordered right"><xsl:value-of select="@NumCompte"/></td>
                                        <td class="bordered right"><xsl:value-of select="@ComptePEC"/></td>
                                    </xsl:otherwise>
                                </xsl:choose>
                            </xsl:when>
                            <xsl:otherwise>
                                <td class="bordureRight"><xsl:attribute name="id">padding<xsl:value-of select="@added:Padding"/></xsl:attribute><xsl:value-of select="@NatureProduit"/></td>
                                <td class="bordureRight">&#160;</td>
                                <td class="bordureRight">&#160;</td>
                                <td class="bordureRight">&#160;</td>
                                <td class="bordureRight">&#160;</td>
                                <td class="bordureRight">&#160;</td>
                                <td class="bordureRight">&#160;</td>
                                <td class="bordureRight">&#160;</td>
                                <td class="bordureRight">&#160;</td>
                                <td class="bordureRight">&#160;</td>
                                <td class="bordureRight">&#160;</td>
                                <td class="bordureRight">&#160;</td>
                            </xsl:otherwise>
                        </xsl:choose>
                    </tr>
                </xsl:for-each>
            </tbody>
        </table>
    </xsl:template>
  
</xsl:transform>
